import request from 'supertest'
import { generateSimpleViteManifest, mockApp, mockConfig, mockFetch } from './util.js'
import fs from 'fs'
import { expect } from 'chai'
import * as td from 'testdouble'
import { Response } from 'node-fetch'

const image = fs.readFileSync('./spec/media/image.png')
const imageStat = fs.statSync('./spec/media/image.png')

describe('File caching service', function () {
  let app

  before(async function () {
    mockConfig({ urls: ['http://ui-server/'] })
    mockFetch({
      'http://ui-server': {
        '/manifest.json': generateSimpleViteManifest({
          'example.js': { imports: ['test.txt'] },
          'test.txt': { },
          'main.css': {},
          'index.html': {
            file: 'index.html.js',
            isEntry: true,
            imports: ['example.js'],
            css: ['main.css']
          },
          'image.png': {}
        }),
        '/example.js': () => new Response('this is example', { headers: { 'content-type': 'application/javascript' } }),
        '/test.txt': () => new Response('this is test', { headers: { 'content-type': 'text/plain' } }),
        '/index.html.js': () => new Response('this is index.html.js', { headers: { 'content-type': 'application/javascript' } }),
        '/index.html': () => new Response('<html><head></head><body>it\'s me</body></html>', { headers: { 'content-type': 'text/html' } }),
        '/main.css': () => new Response('.foo { color: #000; }', { headers: { 'content-type': 'text/css' } }),
        '/favicon.ico': 'not really a favicon, though',
        '/image.png': () => {
          return new Response(image, {
            headers: {
              'Content-Type': 'image/png',
              'Content-Length': imageStat.size
            }
          })
        }
      }
    })
    app = await mockApp()
  })

  after(function () {
    td.reset()
  })

  it('serves files defined in manifest.json file', async function () {
    const response = await request(app).get('/example.js')
    expect(response.statusCode).to.equal(200)
    expect(response.headers['content-type']).to.equal('application/javascript')
    expect(response.text).to.equal('this is example')
    // expect(response.headers['content-security-policy']).to.contain('sha256-NzZhMTE2Njc2YTgyNTZmZTdlZGVjZDU3YTNmYzRjNmM1OWZkMTI2NjRkYzZmMWM3YTkwMGU3ZTdhNDlhZmVlMwo=')
    const response2 = await request(app).get('/test.txt')
    expect(response2.statusCode).to.equal(200)
    expect(response2.headers['content-type']).to.equal('text/plain')
    expect(response2.text).to.equal('this is test')
  })

  it('serves css files', async function () {
    const response = await request(app).get('/main.css')
    expect(response.statusCode).to.equal(200)
    expect(response.headers['content-type']).to.equal('text/css')
    // expect(response.headers['content-security-policy']).to.contain('sha256-YjRiYWRlYTVhYmM5ZTZkNjE2ZGM4YjcwZWRlNzUxMmU0YjgxY2UxMWExOTI2ZjM1NzM1M2Y2MWJjNmUwMmZjMwo=')
  })

  it('serves / as index.html', async function () {
    const response = await request(app).get('/')
    expect(response.statusCode).to.equal(200)
    expect(response.headers['content-type']).to.equal('text/html')
    expect(response.text).to.equal('<html><head></head><body>it\'s me</body></html>')
  })

  it('adds / to dependencies', async function () {
    const response = await request(app).get('/dependencies')
    expect(response.statusCode).to.equal(200)
    const deps = JSON.parse(response.text)
    expect(deps['/']).to.deep.equal([])
  })

  it('directly fetches files not referenced in manifest.json files from the upstream servers', async function () {
    const response = await request(app).get('/favicon.ico')
    expect(response.statusCode).to.equal(200)
    expect(response.text).to.equal('not really a favicon, though')
  })

  it('returns 404 if file can not be resolved', async function () {
    const response = await request(app).get('/unknown-file.txt')
    expect(response.statusCode).to.equal(404)
  })

  it('serves binary files', async function () {
    const response = await request(app).get('/image.png')
    expect(response.statusCode).to.equal(200)
    expect(response.body.length === imageStat.size)
  })
})
